# Copyright (c) 2019 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import command_completion_service_pb2 as com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2


class CommandCompletionServiceStub(object):
  """Allows clients to observe the status of their submissions.
  Commands may be submitted via the Command Submission Service.
  The on-ledger effects of their submissions are disclosed by the Transaction Service.
  Commands may fail in 4 distinct manners:

  1. ``INVALID_PARAMETER`` gRPC error on malformed payloads and missing required fields.
  2. Failure communicated in the gRPC error.
  3. Failure communicated in a Completion.
  4. A Checkpoint with ``record_time`` > command ``mrt`` arrives through the Completion Stream, and the command's Completion was not visible before. In this case the command is lost.

  Clients that do not receive a successful completion about their submission MUST NOT assume that it was successful.
  Clients SHOULD subscribe to the CompletionStream before starting to submit commands to prevent race conditions.

  Interprocess tracing of command submissions may be achieved via Zipkin by filling out the ``trace_context`` field.
  The server will return a child context of the submitted one, (or a new one if the context was missing) on both the Completion and Transaction streams.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.CompletionStream = channel.unary_stream(
        '/com.digitalasset.ledger.api.v1.CommandCompletionService/CompletionStream',
        request_serializer=com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionStreamRequest.SerializeToString,
        response_deserializer=com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionStreamResponse.FromString,
        )
    self.CompletionEnd = channel.unary_unary(
        '/com.digitalasset.ledger.api.v1.CommandCompletionService/CompletionEnd',
        request_serializer=com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionEndRequest.SerializeToString,
        response_deserializer=com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionEndResponse.FromString,
        )


class CommandCompletionServiceServicer(object):
  """Allows clients to observe the status of their submissions.
  Commands may be submitted via the Command Submission Service.
  The on-ledger effects of their submissions are disclosed by the Transaction Service.
  Commands may fail in 4 distinct manners:

  1. ``INVALID_PARAMETER`` gRPC error on malformed payloads and missing required fields.
  2. Failure communicated in the gRPC error.
  3. Failure communicated in a Completion.
  4. A Checkpoint with ``record_time`` > command ``mrt`` arrives through the Completion Stream, and the command's Completion was not visible before. In this case the command is lost.

  Clients that do not receive a successful completion about their submission MUST NOT assume that it was successful.
  Clients SHOULD subscribe to the CompletionStream before starting to submit commands to prevent race conditions.

  Interprocess tracing of command submissions may be achieved via Zipkin by filling out the ``trace_context`` field.
  The server will return a child context of the submitted one, (or a new one if the context was missing) on both the Completion and Transaction streams.
  """

  def CompletionStream(self, request, context):
    """Subscribe to command completion events.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def CompletionEnd(self, request, context):
    """Returns the offset after the latest completion.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_CommandCompletionServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'CompletionStream': grpc.unary_stream_rpc_method_handler(
          servicer.CompletionStream,
          request_deserializer=com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionStreamRequest.FromString,
          response_serializer=com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionStreamResponse.SerializeToString,
      ),
      'CompletionEnd': grpc.unary_unary_rpc_method_handler(
          servicer.CompletionEnd,
          request_deserializer=com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionEndRequest.FromString,
          response_serializer=com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionEndResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'com.digitalasset.ledger.api.v1.CommandCompletionService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# Copyright (c) 2019 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from argparse import ArgumentParser

from .. import LOG
from ._base import CliCommand
from ..plugins import LedgerCapturePlugin
from ..client.manager import LedgerClientManager
from ..client.config import configure_parser, get_config


class TailCommand(CliCommand):
    name = 'tail'

    def parser(self) -> ArgumentParser:
        arg_parser = ArgumentParser('dazl tail')

        configure_parser(arg_parser)
        arg_parser.add_argument('--config', help='path to a YAML config file')
        arg_parser.add_argument('--format', '--fmt', '-F', type=str,
                                default=LedgerCapturePlugin.DEFAULT_FORMATTER_NAME)
        return arg_parser

    def execute(self, args) -> None:
        LOG.debug('Executing a tail...')

        config = get_config(args)
        with LedgerClientManager(config) as mgr:
            mgr.run_forever()



# Copyright (c) 2019 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/digitalasset/ledger/api/v1/command_completion_service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import completion_pb2 as com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_completion__pb2
from . import ledger_offset_pb2 as com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_ledger__offset__pb2
from . import trace_context_pb2 as com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='com/digitalasset/ledger/api/v1/command_completion_service.proto',
  package='com.digitalasset.ledger.api.v1',
  syntax='proto3',
  serialized_options=_b('\n\036com.digitalasset.ledger.api.v1B\"CommandCompletionServiceOuterClass'),
  serialized_pb=_b('\n?com/digitalasset/ledger/api/v1/command_completion_service.proto\x12\x1e\x63om.digitalasset.ledger.api.v1\x1a/com/digitalasset/ledger/api/v1/completion.proto\x1a\x32\x63om/digitalasset/ledger/api/v1/ledger_offset.proto\x1a\x32\x63om/digitalasset/ledger/api/v1/trace_context.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x93\x01\n\x17\x43ompletionStreamRequest\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x16\n\x0e\x61pplication_id\x18\x02 \x01(\t\x12\x0f\n\x07parties\x18\x03 \x03(\t\x12<\n\x06offset\x18\x04 \x01(\x0b\x32,.com.digitalasset.ledger.api.v1.LedgerOffset\"\x9b\x01\n\x18\x43ompletionStreamResponse\x12>\n\ncheckpoint\x18\x01 \x01(\x0b\x32*.com.digitalasset.ledger.api.v1.Checkpoint\x12?\n\x0b\x63ompletions\x18\x02 \x03(\x0b\x32*.com.digitalasset.ledger.api.v1.Completion\"{\n\nCheckpoint\x12/\n\x0brecord_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12<\n\x06offset\x18\x02 \x01(\x0b\x32,.com.digitalasset.ledger.api.v1.LedgerOffset\"o\n\x14\x43ompletionEndRequest\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x44\n\rtrace_context\x18\xe8\x07 \x01(\x0b\x32,.com.digitalasset.ledger.api.v1.TraceContext\"U\n\x15\x43ompletionEndResponse\x12<\n\x06offset\x18\x01 \x01(\x0b\x32,.com.digitalasset.ledger.api.v1.LedgerOffset2\xa2\x02\n\x18\x43ommandCompletionService\x12\x87\x01\n\x10\x43ompletionStream\x12\x37.com.digitalasset.ledger.api.v1.CompletionStreamRequest\x1a\x38.com.digitalasset.ledger.api.v1.CompletionStreamResponse0\x01\x12|\n\rCompletionEnd\x12\x34.com.digitalasset.ledger.api.v1.CompletionEndRequest\x1a\x35.com.digitalasset.ledger.api.v1.CompletionEndResponseBD\n\x1e\x63om.digitalasset.ledger.api.v1B\"CommandCompletionServiceOuterClassb\x06proto3')
  ,
  dependencies=[com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_completion__pb2.DESCRIPTOR,com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_ledger__offset__pb2.DESCRIPTOR,com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])




_COMPLETIONSTREAMREQUEST = _descriptor.Descriptor(
  name='CompletionStreamRequest',
  full_name='com.digitalasset.ledger.api.v1.CompletionStreamRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ledger_id', full_name='com.digitalasset.ledger.api.v1.CompletionStreamRequest.ledger_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='application_id', full_name='com.digitalasset.ledger.api.v1.CompletionStreamRequest.application_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='parties', full_name='com.digitalasset.ledger.api.v1.CompletionStreamRequest.parties', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='offset', full_name='com.digitalasset.ledger.api.v1.CompletionStreamRequest.offset', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=286,
  serialized_end=433,
)


_COMPLETIONSTREAMRESPONSE = _descriptor.Descriptor(
  name='CompletionStreamResponse',
  full_name='com.digitalasset.ledger.api.v1.CompletionStreamResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='checkpoint', full_name='com.digitalasset.ledger.api.v1.CompletionStreamResponse.checkpoint', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='completions', full_name='com.digitalasset.ledger.api.v1.CompletionStreamResponse.completions', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=436,
  serialized_end=591,
)


_CHECKPOINT = _descriptor.Descriptor(
  name='Checkpoint',
  full_name='com.digitalasset.ledger.api.v1.Checkpoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='record_time', full_name='com.digitalasset.ledger.api.v1.Checkpoint.record_time', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='offset', full_name='com.digitalasset.ledger.api.v1.Checkpoint.offset', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=593,
  serialized_end=716,
)


_COMPLETIONENDREQUEST = _descriptor.Descriptor(
  name='CompletionEndRequest',
  full_name='com.digitalasset.ledger.api.v1.CompletionEndRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ledger_id', full_name='com.digitalasset.ledger.api.v1.CompletionEndRequest.ledger_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='trace_context', full_name='com.digitalasset.ledger.api.v1.CompletionEndRequest.trace_context', index=1,
      number=1000, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=718,
  serialized_end=829,
)


_COMPLETIONENDRESPONSE = _descriptor.Descriptor(
  name='CompletionEndResponse',
  full_name='com.digitalasset.ledger.api.v1.CompletionEndResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='offset', full_name='com.digitalasset.ledger.api.v1.CompletionEndResponse.offset', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=831,
  serialized_end=916,
)

_COMPLETIONSTREAMREQUEST.fields_by_name['offset'].message_type = com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_ledger__offset__pb2._LEDGEROFFSET
_COMPLETIONSTREAMRESPONSE.fields_by_name['checkpoint'].message_type = _CHECKPOINT
_COMPLETIONSTREAMRESPONSE.fields_by_name['completions'].message_type = com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_completion__pb2._COMPLETION
_CHECKPOINT.fields_by_name['record_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_CHECKPOINT.fields_by_name['offset'].message_type = com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_ledger__offset__pb2._LEDGEROFFSET
_COMPLETIONENDREQUEST.fields_by_name['trace_context'].message_type = com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2._TRACECONTEXT
_COMPLETIONENDRESPONSE.fields_by_name['offset'].message_type = com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_ledger__offset__pb2._LEDGEROFFSET
DESCRIPTOR.message_types_by_name['CompletionStreamRequest'] = _COMPLETIONSTREAMREQUEST
DESCRIPTOR.message_types_by_name['CompletionStreamResponse'] = _COMPLETIONSTREAMRESPONSE
DESCRIPTOR.message_types_by_name['Checkpoint'] = _CHECKPOINT
DESCRIPTOR.message_types_by_name['CompletionEndRequest'] = _COMPLETIONENDREQUEST
DESCRIPTOR.message_types_by_name['CompletionEndResponse'] = _COMPLETIONENDRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CompletionStreamRequest = _reflection.GeneratedProtocolMessageType('CompletionStreamRequest', (_message.Message,), dict(
  DESCRIPTOR = _COMPLETIONSTREAMREQUEST,
  __module__ = 'com.digitalasset.ledger.api.v1.command_completion_service_pb2'
  # @@protoc_insertion_point(class_scope:com.digitalasset.ledger.api.v1.CompletionStreamRequest)
  ))
_sym_db.RegisterMessage(CompletionStreamRequest)

CompletionStreamResponse = _reflection.GeneratedProtocolMessageType('CompletionStreamResponse', (_message.Message,), dict(
  DESCRIPTOR = _COMPLETIONSTREAMRESPONSE,
  __module__ = 'com.digitalasset.ledger.api.v1.command_completion_service_pb2'
  # @@protoc_insertion_point(class_scope:com.digitalasset.ledger.api.v1.CompletionStreamResponse)
  ))
_sym_db.RegisterMessage(CompletionStreamResponse)

Checkpoint = _reflection.GeneratedProtocolMessageType('Checkpoint', (_message.Message,), dict(
  DESCRIPTOR = _CHECKPOINT,
  __module__ = 'com.digitalasset.ledger.api.v1.command_completion_service_pb2'
  # @@protoc_insertion_point(class_scope:com.digitalasset.ledger.api.v1.Checkpoint)
  ))
_sym_db.RegisterMessage(Checkpoint)

CompletionEndRequest = _reflection.GeneratedProtocolMessageType('CompletionEndRequest', (_message.Message,), dict(
  DESCRIPTOR = _COMPLETIONENDREQUEST,
  __module__ = 'com.digitalasset.ledger.api.v1.command_completion_service_pb2'
  # @@protoc_insertion_point(class_scope:com.digitalasset.ledger.api.v1.CompletionEndRequest)
  ))
_sym_db.RegisterMessage(CompletionEndRequest)

CompletionEndResponse = _reflection.GeneratedProtocolMessageType('CompletionEndResponse', (_message.Message,), dict(
  DESCRIPTOR = _COMPLETIONENDRESPONSE,
  __module__ = 'com.digitalasset.ledger.api.v1.command_completion_service_pb2'
  # @@protoc_insertion_point(class_scope:com.digitalasset.ledger.api.v1.CompletionEndResponse)
  ))
_sym_db.RegisterMessage(CompletionEndResponse)


DESCRIPTOR._options = None

_COMMANDCOMPLETIONSERVICE = _descriptor.ServiceDescriptor(
  name='CommandCompletionService',
  full_name='com.digitalasset.ledger.api.v1.CommandCompletionService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=919,
  serialized_end=1209,
  methods=[
  _descriptor.MethodDescriptor(
    name='CompletionStream',
    full_name='com.digitalasset.ledger.api.v1.CommandCompletionService.CompletionStream',
    index=0,
    containing_service=None,
    input_type=_COMPLETIONSTREAMREQUEST,
    output_type=_COMPLETIONSTREAMRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CompletionEnd',
    full_name='com.digitalasset.ledger.api.v1.CommandCompletionService.CompletionEnd',
    index=1,
    containing_service=None,
    input_type=_COMPLETIONENDREQUEST,
    output_type=_COMPLETIONENDRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_COMMANDCOMPLETIONSERVICE)

DESCRIPTOR.services_by_name['CommandCompletionService'] = _COMMANDCOMPLETIONSERVICE

# @@protoc_insertion_point(module_scope)



# Copyright (c) 2019 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/digitalasset/ledger/api/v1/value.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='com/digitalasset/ledger/api/v1/value.proto',
  package='com.digitalasset.ledger.api.v1',
  syntax='proto3',
  serialized_options=_b('\n\036com.digitalasset.ledger.api.v1B\017ValueOuterClass'),
  serialized_pb=_b('\n*com/digitalasset/ledger/api/v1/value.proto\x12\x1e\x63om.digitalasset.ledger.api.v1\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto\"\xed\x03\n\x05Value\x12\x38\n\x06record\x18\x01 \x01(\x0b\x32&.com.digitalasset.ledger.api.v1.RecordH\x00\x12:\n\x07variant\x18\x02 \x01(\x0b\x32\'.com.digitalasset.ledger.api.v1.VariantH\x00\x12\x15\n\x0b\x63ontract_id\x18\x03 \x01(\tH\x00\x12\x34\n\x04list\x18\x04 \x01(\x0b\x32$.com.digitalasset.ledger.api.v1.ListH\x00\x12\x13\n\x05int64\x18\x05 \x01(\x12\x42\x02\x30\x01H\x00\x12\x11\n\x07\x64\x65\x63imal\x18\x06 \x01(\tH\x00\x12\x0e\n\x04text\x18\x08 \x01(\tH\x00\x12\x17\n\ttimestamp\x18\t \x01(\x10\x42\x02\x30\x01H\x00\x12\x0f\n\x05party\x18\x0b \x01(\tH\x00\x12\x0e\n\x04\x62ool\x18\x0c \x01(\x08H\x00\x12&\n\x04unit\x18\r \x01(\x0b\x32\x16.google.protobuf.EmptyH\x00\x12\x0e\n\x04\x64\x61te\x18\x0e \x01(\x05H\x00\x12<\n\x08optional\x18\x0f \x01(\x0b\x32(.com.digitalasset.ledger.api.v1.OptionalH\x00\x12\x32\n\x03map\x18\x10 \x01(\x0b\x32#.com.digitalasset.ledger.api.v1.MapH\x00\x42\x05\n\x03Sum\"\x84\x01\n\x06Record\x12=\n\trecord_id\x18\x01 \x01(\x0b\x32*.com.digitalasset.ledger.api.v1.Identifier\x12;\n\x06\x66ields\x18\x02 \x03(\x0b\x32+.com.digitalasset.ledger.api.v1.RecordField\"R\n\x0bRecordField\x12\r\n\x05label\x18\x01 \x01(\t\x12\x34\n\x05value\x18\x02 \x01(\x0b\x32%.com.digitalasset.ledger.api.v1.Value\"\\\n\nIdentifier\x12\x12\n\npackage_id\x18\x01 \x01(\t\x12\x10\n\x04name\x18\x02 \x01(\tB\x02\x18\x01\x12\x13\n\x0bmodule_name\x18\x03 \x01(\t\x12\x13\n\x0b\x65ntity_name\x18\x04 \x01(\t\"\x94\x01\n\x07Variant\x12>\n\nvariant_id\x18\x01 \x01(\x0b\x32*.com.digitalasset.ledger.api.v1.Identifier\x12\x13\n\x0b\x63onstructor\x18\x02 \x01(\t\x12\x34\n\x05value\x18\x03 \x01(\x0b\x32%.com.digitalasset.ledger.api.v1.Value\"?\n\x04List\x12\x37\n\x08\x65lements\x18\x01 \x03(\x0b\x32%.com.digitalasset.ledger.api.v1.Value\"@\n\x08Optional\x12\x34\n\x05value\x18\x01 \x01(\x0b\x32%.com.digitalasset.ledger.api.v1.Value\"\x8d\x01\n\x03Map\x12:\n\x07\x65ntries\x18\x01 \x03(\x0b\x32).com.digitalasset.ledger.api.v1.Map.Entry\x1aJ\n\x05\x45ntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x34\n\x05value\x18\x02 \x01(\x0b\x32%.com.digitalasset.ledger.api.v1.ValueB1\n\x1e\x63om.digitalasset.ledger.api.v1B\x0fValueOuterClassb\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,])




_VALUE = _descriptor.Descriptor(
  name='Value',
  full_name='com.digitalasset.ledger.api.v1.Value',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='record', full_name='com.digitalasset.ledger.api.v1.Value.record', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='variant', full_name='com.digitalasset.ledger.api.v1.Value.variant', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contract_id', full_name='com.digitalasset.ledger.api.v1.Value.contract_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='list', full_name='com.digitalasset.ledger.api.v1.Value.list', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='int64', full_name='com.digitalasset.ledger.api.v1.Value.int64', index=4,
      number=5, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('0\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='decimal', full_name='com.digitalasset.ledger.api.v1.Value.decimal', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='text', full_name='com.digitalasset.ledger.api.v1.Value.text', index=6,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='com.digitalasset.ledger.api.v1.Value.timestamp', index=7,
      number=9, type=16, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('0\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='party', full_name='com.digitalasset.ledger.api.v1.Value.party', index=8,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bool', full_name='com.digitalasset.ledger.api.v1.Value.bool', index=9,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unit', full_name='com.digitalasset.ledger.api.v1.Value.unit', index=10,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='date', full_name='com.digitalasset.ledger.api.v1.Value.date', index=11,
      number=14, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='optional', full_name='com.digitalasset.ledger.api.v1.Value.optional', index=12,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='map', full_name='com.digitalasset.ledger.api.v1.Value.map', index=13,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='Sum', full_name='com.digitalasset.ledger.api.v1.Value.Sum',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=173,
  serialized_end=666,
)


_RECORD = _descriptor.Descriptor(
  name='Record',
  full_name='com.digitalasset.ledger.api.v1.Record',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='record_id', full_name='com.digitalasset.ledger.api.v1.Record.record_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fields', full_name='com.digitalasset.ledger.api.v1.Record.fields', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=669,
  serialized_end=801,
)


_RECORDFIELD = _descriptor.Descriptor(
  name='RecordField',
  full_name='com.digitalasset.ledger.api.v1.RecordField',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='label', full_name='com.digitalasset.ledger.api.v1.RecordField.label', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='com.digitalasset.ledger.api.v1.RecordField.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=803,
  serialized_end=885,
)


_IDENTIFIER = _descriptor.Descriptor(
  name='Identifier',
  full_name='com.digitalasset.ledger.api.v1.Identifier',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='package_id', full_name='com.digitalasset.ledger.api.v1.Identifier.package_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='com.digitalasset.ledger.api.v1.Identifier.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\030\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='module_name', full_name='com.digitalasset.ledger.api.v1.Identifier.module_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='entity_name', full_name='com.digitalasset.ledger.api.v1.Identifier.entity_name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=887,
  serialized_end=979,
)


_VARIANT = _descriptor.Descriptor(
  name='Variant',
  full_name='com.digitalasset.ledger.api.v1.Variant',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='variant_id', full_name='com.digitalasset.ledger.api.v1.Variant.variant_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='constructor', full_name='com.digitalasset.ledger.api.v1.Variant.constructor', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='com.digitalasset.ledger.api.v1.Variant.value', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=982,
  serialized_end=1130,
)


_LIST = _descriptor.Descriptor(
  name='List',
  full_name='com.digitalasset.ledger.api.v1.List',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='elements', full_name='com.digitalasset.ledger.api.v1.List.elements', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1132,
  serialized_end=1195,
)


_OPTIONAL = _descriptor.Descriptor(
  name='Optional',
  full_name='com.digitalasset.ledger.api.v1.Optional',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='com.digitalasset.ledger.api.v1.Optional.value', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1197,
  serialized_end=1261,
)


_MAP_ENTRY = _descriptor.Descriptor(
  name='Entry',
  full_name='com.digitalasset.ledger.api.v1.Map.Entry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='com.digitalasset.ledger.api.v1.Map.Entry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='com.digitalasset.ledger.api.v1.Map.Entry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1331,
  serialized_end=1405,
)

_MAP = _descriptor.Descriptor(
  name='Map',
  full_name='com.digitalasset.ledger.api.v1.Map',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='entries', full_name='com.digitalasset.ledger.api.v1.Map.entries', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_MAP_ENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1264,
  serialized_end=1405,
)

_VALUE.fields_by_name['record'].message_type = _RECORD
_VALUE.fields_by_name['variant'].message_type = _VARIANT
_VALUE.fields_by_name['list'].message_type = _LIST
_VALUE.fields_by_name['unit'].message_type = google_dot_protobuf_dot_empty__pb2._EMPTY
_VALUE.fields_by_name['optional'].message_type = _OPTIONAL
_VALUE.fields_by_name['map'].message_type = _MAP
_VALUE.oneofs_by_name['Sum'].fields.append(
  _VALUE.fields_by_name['record'])
_VALUE.fields_by_name['record'].containing_oneof = _VALUE.oneofs_by_name['Sum']
_VALUE.oneofs_by_name['Sum'].fields.append(
  _VALUE.fields_by_name['variant'])
_VALUE.fields_by_name['variant'].containing_oneof = _VALUE.oneofs_by_name['Sum']
_VALUE.oneofs_by_name['Sum'].fields.append(
  _VALUE.fields_by_name['contract_id'])
_VALUE.fields_by_name['contract_id'].containing_oneof = _VALUE.oneofs_by_name['Sum']
_VALUE.oneofs_by_name['Sum'].fields.append(
  _VALUE.fields_by_name['list'])
_VALUE.fields_by_name['list'].containing_oneof = _VALUE.oneofs_by_name['Sum']
_VALUE.oneofs_by_name['Sum'].fields.append(
  _VALUE.fields_by_name['int64'])
_VALUE.fields_by_name['int64'].containing_oneof = _VALUE.oneofs_by_name['Sum']
_VALUE.oneofs_by_name['Sum'].fields.append(
  _VALUE.fields_by_name['decimal'])
_VALUE.fields_by_name['decimal'].containing_oneof = _VALUE.oneofs_by_name['Sum']
_VALUE.oneofs_by_name['Sum'].fields.append(
  _VALUE.fields_by_name['text'])
_VALUE.fields_by_name['text'].containing_oneof = _VALUE.oneofs_by_name['Sum']
_VALUE.oneofs_by_name['Sum'].fields.append(
  _VALUE.fields_by_name['timestamp'])
_VALUE.fields_by_name['timestamp'].containing_oneof = _VALUE.oneofs_by_name['Sum']
_VALUE.oneofs_by_name['Sum'].fields.append(
  _VALUE.fields_by_name['party'])
_VALUE.fields_by_name['party'].containing_oneof = _VALUE.oneofs_by_name['Sum']
_VALUE.oneofs_by_name['Sum'].fields.append(
  _VALUE.fields_by_name['bool'])
_VALUE.fields_by_name['bool'].containing_oneof = _VALUE.oneofs_by_name['Sum']
_VALUE.oneofs_by_name['Sum'].fields.append(
  _VALUE.fields_by_name['unit'])
_VALUE.fields_by_name['unit'].containing_oneof = _VALUE.oneofs_by_name['Sum']
_VALUE.oneofs_by_name['Sum'].fields.append(
  _VALUE.fields_by_name['date'])
_VALUE.fields_by_name['date'].containing_oneof = _VALUE.oneofs_by_name['Sum']
_VALUE.oneofs_by_name['Sum'].fields.append(
  _VALUE.fields_by_name['optional'])
_VALUE.fields_by_name['optional'].containing_oneof = _VALUE.oneofs_by_name['Sum']
_VALUE.oneofs_by_name['Sum'].fields.append(
  _VALUE.fields_by_name['map'])
_VALUE.fields_by_name['map'].containing_oneof = _VALUE.oneofs_by_name['Sum']
_RECORD.fields_by_name['record_id'].message_type = _IDENTIFIER
_RECORD.fields_by_name['fields'].message_type = _RECORDFIELD
_RECORDFIELD.fields_by_name['value'].message_type = _VALUE
_VARIANT.fields_by_name['variant_id'].message_type = _IDENTIFIER
_VARIANT.fields_by_name['value'].message_type = _VALUE
_LIST.fields_by_name['elements'].message_type = _VALUE
_OPTIONAL.fields_by_name['value'].message_type = _VALUE
_MAP_ENTRY.fields_by_name['value'].message_type = _VALUE
_MAP_ENTRY.containing_type = _MAP
_MAP.fields_by_name['entries'].message_type = _MAP_ENTRY
DESCRIPTOR.message_types_by_name['Value'] = _VALUE
DESCRIPTOR.message_types_by_name['Record'] = _RECORD
DESCRIPTOR.message_types_by_name['RecordField'] = _RECORDFIELD
DESCRIPTOR.message_types_by_name['Identifier'] = _IDENTIFIER
DESCRIPTOR.message_types_by_name['Variant'] = _VARIANT
DESCRIPTOR.message_types_by_name['List'] = _LIST
DESCRIPTOR.message_types_by_name['Optional'] = _OPTIONAL
DESCRIPTOR.message_types_by_name['Map'] = _MAP
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Value = _reflection.GeneratedProtocolMessageType('Value', (_message.Message,), dict(
  DESCRIPTOR = _VALUE,
  __module__ = 'com.digitalasset.ledger.api.v1.value_pb2'
  # @@protoc_insertion_point(class_scope:com.digitalasset.ledger.api.v1.Value)
  ))
_sym_db.RegisterMessage(Value)

Record = _reflection.GeneratedProtocolMessageType('Record', (_message.Message,), dict(
  DESCRIPTOR = _RECORD,
  __module__ = 'com.digitalasset.ledger.api.v1.value_pb2'
  # @@protoc_insertion_point(class_scope:com.digitalasset.ledger.api.v1.Record)
  ))
_sym_db.RegisterMessage(Record)

RecordField = _reflection.GeneratedProtocolMessageType('RecordField', (_message.Message,), dict(
  DESCRIPTOR = _RECORDFIELD,
  __module__ = 'com.digitalasset.ledger.api.v1.value_pb2'
  # @@protoc_insertion_point(class_scope:com.digitalasset.ledger.api.v1.RecordField)
  ))
_sym_db.RegisterMessage(RecordField)

Identifier = _reflection.GeneratedProtocolMessageType('Identifier', (_message.Message,), dict(
  DESCRIPTOR = _IDENTIFIER,
  __module__ = 'com.digitalasset.ledger.api.v1.value_pb2'
  # @@protoc_insertion_point(class_scope:com.digitalasset.ledger.api.v1.Identifier)
  ))
_sym_db.RegisterMessage(Identifier)

Variant = _reflection.GeneratedProtocolMessageType('Variant', (_message.Message,), dict(
  DESCRIPTOR = _VARIANT,
  __module__ = 'com.digitalasset.ledger.api.v1.value_pb2'
  # @@protoc_insertion_point(class_scope:com.digitalasset.ledger.api.v1.Variant)
  ))
_sym_db.RegisterMessage(Variant)

List = _reflection.GeneratedProtocolMessageType('List', (_message.Message,), dict(
  DESCRIPTOR = _LIST,
  __module__ = 'com.digitalasset.ledger.api.v1.value_pb2'
  # @@protoc_insertion_point(class_scope:com.digitalasset.ledger.api.v1.List)
  ))
_sym_db.RegisterMessage(List)

Optional = _reflection.GeneratedProtocolMessageType('Optional', (_message.Message,), dict(
  DESCRIPTOR = _OPTIONAL,
  __module__ = 'com.digitalasset.ledger.api.v1.value_pb2'
  # @@protoc_insertion_point(class_scope:com.digitalasset.ledger.api.v1.Optional)
  ))
_sym_db.RegisterMessage(Optional)

Map = _reflection.GeneratedProtocolMessageType('Map', (_message.Message,), dict(

  Entry = _reflection.GeneratedProtocolMessageType('Entry', (_message.Message,), dict(
    DESCRIPTOR = _MAP_ENTRY,
    __module__ = 'com.digitalasset.ledger.api.v1.value_pb2'
    # @@protoc_insertion_point(class_scope:com.digitalasset.ledger.api.v1.Map.Entry)
    ))
  ,
  DESCRIPTOR = _MAP,
  __module__ = 'com.digitalasset.ledger.api.v1.value_pb2'
  # @@protoc_insertion_point(class_scope:com.digitalasset.ledger.api.v1.Map)
  ))
_sym_db.RegisterMessage(Map)
_sym_db.RegisterMessage(Map.Entry)


DESCRIPTOR._options = None
_VALUE.fields_by_name['int64']._options = None
_VALUE.fields_by_name['timestamp']._options = None
_IDENTIFIER.fields_by_name['name']._options = None
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2019 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
This module contains a utility visitor for evaluating an expression tree.
"""

from typing import Any, Callable

from ._rewriter import RewriteContext
from .expr_impl import CreateStatementExpr, ExerciseStatementExpr, UpdateBlockExpr, \
    ConstantExpr, GetContractIdExpr, GetContractDataExpr, \
    TemplateSelectExpr, TemplateFieldExpr, AppExpr
from ..model.writing import CreateCommand, ExerciseCommand


def generate_lambda(expr):
    """
    Produces a function that takes a parameter and produces a result. The specific signature of the
    function varies on the specific kind of node passed in.
    """
    return lambda_generator.rewrite(expr)


# Visitor that produces callbacks based on the passed-in expression tree.
lambda_generator = RewriteContext()


@lambda_generator.register(ConstantExpr, recurse=False)
def constant(expr: ConstantExpr) -> Callable[[Any], Any]:
    """
    Produce a function that takes an arbitrary object and produces a constant value. The
    incoming parameter is ignored.
    """
    return lambda a: expr.value


@lambda_generator.register(AppExpr, recurse=False)
def _app(expr: AppExpr):
    arg_fns = tuple(lambda_generator.rewrite(arg) for arg in expr.args)
    return lambda a: expr.func.impl(*(arg_fn(a) for arg_fn in arg_fns))


@lambda_generator.register(GetContractIdExpr, recurse=False)
def get_cid(expr: GetContractIdExpr):
    fn = lambda_generator.rewrite(expr.expression)
    return lambda a: fn(a)[0]


@lambda_generator.register(GetContractDataExpr, recurse=False)
def get_cdata(expr: GetContractDataExpr):
    fn = lambda_generator.rewrite(expr.expression)
    return lambda a: fn(a)[1]


@lambda_generator.register(TemplateFieldExpr, recurse=False)
def field(expr: TemplateFieldExpr):
    """ Return a callback that extracts a field value. """
    from operator import getitem

    def _resolve_value(obj):
        try:
            return getitem(obj, expr.field_name)
        except (KeyError, TypeError):
            # TODO: This implementation needs to be fixed
            return None

    fn = lambda_generator.rewrite(expr.expression)
    get_field = _resolve_value
    return _compose(fn, get_field)


@lambda_generator.register(TemplateSelectExpr, recurse=False)
def select(expr: TemplateSelectExpr):
    """ Return a callback that extracts `ContractData` from the incoming parameter. """
    if expr.alias is None:
        raise ValueError('trying to create a lambda with an unnamed `let`')
    return lambda entry: entry[expr.alias]


@lambda_generator.register(CreateStatementExpr, recurse=False)
def create(expr: CreateStatementExpr):
    return lambda entry: CreateCommand(
        template=expr.template,
        arguments={k: lambda_generator.rewrite(v)(entry) for k, v in expr.arguments.items()})


@lambda_generator.register(ExerciseStatementExpr, recurse=False)
def exercise(expr: ExerciseStatementExpr):
    def _create_command(entry):
        template_id = None

        # TODO: This awkwardness can be removed as soon as ContractIds gain type parameters
        if isinstance(expr.expression, GetContractIdExpr):
            template_id = expr.expression.expression.template.template_ref

        return ExerciseCommand(
            contract=lambda_generator.rewrite(expr.expression)(entry),
            choice=expr.choice_name,
            arguments={k: lambda_generator.rewrite(v)(entry) for k, v in expr.arguments.items()},
        )#template_id=template_id)

    return _create_command


@lambda_generator.register(UpdateBlockExpr, recurse=False)
def update_block(expr: UpdateBlockExpr):
    statement_fns = tuple(lambda_generator.rewrite(stmt) for stmt in expr.statements)
    return lambda a: [stmt(a) for stmt in statement_fns]


def _compose(*functions):
    """
    Return a function that takes a single argument that is the result of feeding the result of
    one object into the next function.
    """
    def _impl(obj):
        for func in functions:
            obj = func(obj)
        return obj
    return _impl




# Copyright (c) 2019 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0


from typing import Callable

from ._base import T
from ..model.types import Type as OldType
from ..util.typing import safe_cast
from .daml_lf_1 import Type, PrimType


STRING = Type(prim=Type.Prim(prim=PrimType.TEXT, args=()))


def var(var_: str) -> 'Type':
    return Type(var=Type.Var(var_, ()))


def match_prim_type(
        prim_type: 'Type.Prim',
        on_unit: 'Callable[[], T]',
        on_bool: 'Callable[[], T]',
        on_int: 'Callable[[], T]',
        on_decimal: 'Callable[[], T]',
        on_text: 'Callable[[], T]',
        on_datetime: 'Callable[[], T]',
        on_timedelta: 'Callable[[], T]',
        on_party: 'Callable[[], T]',
        on_list: 'Callable[[Type], T]',
        on_update: 'Callable[[Type], T]',
        on_scenario: 'Callable[[Type], T]',
        on_date: 'Callable[[], T]',
        on_contract_id: 'Callable[[Type], T]',
        on_optional: 'Callable[[Type], T]',
        on_arrow: 'Callable[[Type, Type], T]',
        on_map: 'Callable[[Type, Type], T]') -> 'T':
    if prim_type.prim == PrimType.UNIT:
        return on_unit()
    elif prim_type.prim == PrimType.BOOL:
        return on_bool()
    elif prim_type.prim == PrimType.INT64:
        return on_int()
    elif prim_type.prim == PrimType.DECIMAL:
        return on_decimal()
    elif prim_type.prim == PrimType.TEXT or prim_type.prim == PrimType.CHAR:
        return on_text()
    elif prim_type.prim == PrimType.TIMESTAMP:
        return on_datetime()
    elif prim_type.prim == PrimType.RELTIME:
        return on_timedelta()
    elif prim_type.prim == PrimType.PARTY:
        return on_party()
    elif prim_type.prim == PrimType.LIST:
        return on_list(prim_type.args[0])
    elif prim_type.prim == PrimType.UPDATE:
        return on_update(prim_type.args[0])
    elif prim_type.prim == PrimType.SCENARIO:
        return on_scenario(prim_type.args[0])
    elif prim_type.prim == PrimType.DATE:
        return on_date()
    elif prim_type.prim == PrimType.CONTRACT_ID:
        return on_contract_id(prim_type.args[0])
    elif prim_type.prim == PrimType.OPTIONAL:
        return on_optional(prim_type.args[0])
    elif prim_type.prim == PrimType.ARROW:
        return on_arrow(prim_type.args[0], prim_type.args[1])
    elif prim_type.prim == PrimType.MAP:
        return on_map(STRING, prim_type.args[0])
    elif prim_type.prim == PrimType.MAP_GENERIC:
        return on_map(prim_type.args[0], prim_type.args[1])
    else:
        raise ValueError(f'undefined PrimType: {prim_type}')


def get_old_type(daml_type: 'Type') -> 'OldType':
    from ..model.types import UnsupportedType

    return safe_cast(Type, daml_type).Sum_match(
        _old_type_var, _old_type_con, _old_type_prim, _old_forall_type,
        lambda tuple_: UnsupportedType('Tuple'))


def _old_type_var(var_: 'Type.Var') -> 'OldType':
    from ..model.types import TypeApp, TypeVariable
    core_type = TypeVariable(var_.var)
    return TypeApp(core_type, [get_old_type(var_) for var_ in var_.args]) if var_.args else core_type


def _old_type_con(con: 'Type.Con') -> 'OldType':
    from ..model.types import TypeApp
    core_type = con.tycon
    return TypeApp(core_type, [get_old_type(var) for var in con.args]) if con.args else core_type


def _old_type_prim(prim: 'Type.Prim') -> 'OldType':
    from ..model.types import UnsupportedType

    core_type = match_prim_type(
        prim,
        _old_scalar_type_unit,
        _old_scalar_type_bool,
        _old_scalar_type_integer,
        _old_scalar_type_decimal,
        _old_scalar_type_text,
        _old_scalar_type_datetime,
        _old_scalar_type_reltime,
        _old_scalar_type_party,
        _old_list_type,
        _old_update_type,
        lambda *args: UnsupportedType('Scenario'),
        _old_scalar_type_date,
        _old_scalar_contract_id_type,
        _old_optional_type,
        lambda *args: UnsupportedType('Arrow'),
        _old_map_type)

    # TODO: This is pretty ugly
    if hasattr(prim, 'internal_type') and prim.internal_type is not None:
        core_type.internal_type = get_old_type(prim.internal_type)

    return core_type


def _old_forall_type(forall: 'Type.Forall') -> 'OldType':
    from ..model.types import ForAllType
    return ForAllType(forall.vars, get_old_type(forall.body))


def _old_scalar_type_unit() -> 'OldType':
    from ..model.types import SCALAR_TYPE_UNIT
    return SCALAR_TYPE_UNIT


def _old_scalar_type_bool() -> 'OldType':
    from ..model.types import SCALAR_TYPE_BOOL
    return SCALAR_TYPE_BOOL


def _old_scalar_type_integer() -> 'OldType':
    from ..model.types import SCALAR_TYPE_INTEGER
    return SCALAR_TYPE_INTEGER


def _old_scalar_type_decimal() -> 'OldType':
    from ..model.types import SCALAR_TYPE_DECIMAL
    return SCALAR_TYPE_DECIMAL


def _old_scalar_type_text() -> 'OldType':
    from ..model.types import SCALAR_TYPE_TEXT
    return SCALAR_TYPE_TEXT


def _old_scalar_type_datetime() -> 'OldType':
    from ..model.types import SCALAR_TYPE_DATETIME
    return SCALAR_TYPE_DATETIME


def _old_scalar_type_reltime() -> 'OldType':
    from ..model.types import SCALAR_TYPE_RELTIME
    return SCALAR_TYPE_RELTIME


def _old_scalar_type_party() -> 'OldType':
    from ..model.types import SCALAR_TYPE_PARTY
    return SCALAR_TYPE_PARTY


def _old_list_type(arg: 'Type') -> 'OldType':
    from ..model.types import ListType
    return ListType(get_old_type(arg))


def _old_update_type(arg: 'Type') -> 'OldType':
    from ..model.types import UpdateType
    return UpdateType(get_old_type(arg))


def _old_scalar_type_date() -> 'OldType':
    from ..model.types import SCALAR_TYPE_DATE
    return SCALAR_TYPE_DATE


def _old_scalar_contract_id_type(arg: 'Type') -> 'OldType':
    from ..model.types import ContractIdType
    return ContractIdType(get_old_type(arg))


def _old_optional_type(arg: 'Type') -> 'OldType':
    from ..model.types import OptionalType
    internal_type = arg._Sum_value.internal_type if hasattr(arg._Sum_value, 'internal_type') \
        else None
    return OptionalType(get_old_type(arg), internal_type=internal_type)


def _old_map_type(key_type: 'Type', value_type: 'Type') -> 'OldType':
    from ..model.types import MapType
    return MapType(get_old_type(key_type), get_old_type(value_type))



# Copyright (c) 2019 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
Methods for serializing domain objects and other things into basic primitive
types over the wire on the REST interface using JSON.
"""

import abc
import json

from datetime import datetime, timezone, date
from decimal import Decimal
from typing import Union, Dict, Any, List

from ... import LOG
from ...model.core import ContractId
from ...model.types import ScalarType, ListType, RecordType, VariantType, \
    ContractIdType, UnsupportedType, as_contract_id, TemplateChoice, MapType
from ...model.writing import CommandPayload, AbstractSerializer, TypeEvaluationContext
from ...util.prim_types import to_boolean, to_date, to_str, to_decimal, decode_variant_dict, \
    to_int, to_datetime, to_timedelta

R = Union[bool, str, date, datetime, int, float, Decimal, List['R'], Dict[str, 'R']]


class BodyDecoder(abc.ABC):
    @abc.abstractmethod
    def parse(self, code, buf):
        """

        :param code: The HTTP status code associated with the body.
        :param buf:
        :return:
        """


class LedgerJSONEncoder(json.JSONEncoder):
    """
    Convert some known Ledger API primitive types into their appropriate JSON
    representations.
    """
    def default(self, o):
        # pylint: disable=E0202
        # bug in pylint https://github.com/PyCQA/pylint/issues/414)
        if isinstance(o, datetime):
            return _datetime_to_api_datetime(o)
        elif isinstance(o, date):
            return _date_to_api_date(o)
        elif isinstance(o, ContractId):
            return o.contract_id
        elif isinstance(o, Decimal):
            return _NoStringDecimal(o)

        return json.JSONEncoder.default(self, o)


def to_api_datetime(obj):
    """
    Converts the object to an ISO8601 datetime string.

    :param obj:
        A datetime. If the datetime is "naive" (no timezone information), it is
        assumed to refer to UTC. If the datetime has timezone information, the
        datetime will be converted to UTC before serializing.
    :return: An ISO8601 string that represents the time in UTC.
    """
    if isinstance(obj, datetime):
        return _datetime_to_api_datetime(obj)
    elif isinstance(obj, str):
        # attempt to understand this string as a datetime
        pass
    return obj


def _date_to_api_date(obj: date) -> str:
    """
    Convert date to a JSON string that represents a date.
    """
    return obj.isoformat()


def _datetime_to_api_datetime(obj: datetime) -> str:
    """
    Convert timezone "aware" dates to UTC.
    """
    if obj.tzinfo is not None and obj.tzinfo.utcoffset(obj) is not None:
        # aware time; translate to UTC
        obj = obj.astimezone(timezone.utc)
    obj = obj.replace(tzinfo=None)
    return obj.isoformat() + 'Z'


class _NoStringDecimal(float):
    def __new__(cls, decimal_value):
        # noinspection PyArgumentList
        return float.__new__(cls, float(decimal_value))

    def __init__(self, decimal_value):
        float.__init__(float(decimal_value))
        self.decimal_value = decimal_value

    def __repr__(self):
        return str(self.decimal_value)


####################################################################################################
# for_json support
####################################################################################################


def timedelta_for_json(value):
    from datetime import timedelta
    if isinstance(value, timedelta):
        # TODO: Implementation
        raise NotImplementedError('reltime are not currently supported')
    # TODO: Some light coercion is probably required here
    return value


def date_for_json(value):
    """
    Return a Python ``date``, and rely on the JSON encoder in
    :class:`LedgerJSONEncoder` to do the appropriate conversion.
    """
    return value


def time_for_json(value):
    """
    Return a Python ``datetime``, and rely on the JSON encoder in
    :mod:`da.protocols.ser_json` to do the appropriate conversion.
    """
    return value


def _weakbound_for_json(value):
    """
    Simply call `for_json` on the underlying value if it exists; otherwise, return the underlying
    value.
    """
    return value.for_json() if hasattr(value, 'for_json') else value


def _define_scalar_formatters():
    """
    Produce a dict whose keys are :class:`ScalarType` and values are methods that can take a
    value and convert them to a JSON representation.
    """
    formatters = {
        "Bool": bool,
        "Char": str,
        "Integer": int,
        "Decimal": Decimal,
        "Text": str,
        "Party": str,
        "RelTime": timedelta_for_json,
        "Date": date_for_json,
        "Time": time_for_json,
    }

    if len(ScalarType.BUILTINS) != len(formatters):
        LOG.error('Programmer error. Formatters: %r, builtins: %r', formatters, ScalarType.BUILTINS)
        raise Exception('library configuration error')
    return {scalar_type: formatters[scalar_type.name] for scalar_type in ScalarType.BUILTINS}


_SCALAR_FORMATTERS = _define_scalar_formatters()


class JsonSerializer(AbstractSerializer[dict, R]):

    ################################################################################################
    # COMMAND serializers
    ################################################################################################

    def serialize_command_request(self, command_payload: CommandPayload) -> dict:
        commands = [self.serialize_command(command) for command in command_payload.commands]
        return dict(
            businessIntent=command_payload.command_id,
            ledgerEffectiveTime=command_payload.ledger_effective_time,
            maximumRecordTime=command_payload.maximum_record_time,
            commands=commands,
            application=command_payload.application_id)

    def serialize_create_command(self, template_type: RecordType, template_args: R) -> dict:
        # the package_id in ModuleRef is a convenient place to
        # stash legacy template IDs in the REST endpoint
        return {'create': {
            'template': template_type.name.module.package_id,
            'arguments': template_args}}

    def serialize_exercise_command(
            self, contract_id: ContractId, choice_info: TemplateChoice, choice_args: R) -> dict:
        return {'exercise': {
            'contract': contract_id.contract_id,
            'choice': choice_info.name,
            'arguments': choice_args}}

    ################################################################################################
    # VALUE serializers
    ################################################################################################

    def serialize_unit(self, context: TypeEvaluationContext, obj: Any) -> R:
        return dict()

    def serialize_bool(self, context: TypeEvaluationContext, obj: Any) -> R:
        return to_boolean(obj)

    def serialize_text(self, context: TypeEvaluationContext, obj: Any) -> R:
        return to_str(obj)

    def serialize_int(self, context: TypeEvaluationContext, obj: Any) -> R:
        return to_int(obj)

    def serialize_decimal(self, context: TypeEvaluationContext, obj: Any) -> R:
        return to_decimal(obj)

    def serialize_party(self, context: TypeEvaluationContext, obj: Any) -> R:
        return to_str(obj)

    def serialize_date(self, context: TypeEvaluationContext, obj: Any) -> R:
        return to_date(obj)

    def serialize_datetime(self, context: TypeEvaluationContext, obj: Any) -> R:
        return to_datetime(obj)

    def serialize_timedelta(self, context: TypeEvaluationContext, obj: Any) -> R:
        return to_timedelta(obj)

    def serialize_contract_id(
            self, context: TypeEvaluationContext, tt: ContractIdType, obj: Any) -> R:
        return as_contract_id(obj, template_id=tt.type_parameter).contract_id

    def serialize_list(self, context: TypeEvaluationContext, tt: ListType, obj: Any) -> R:
        return [self._serialize_dispatch(context.append_path(f'[{i}]'), tt.type_parameter, item)
                for i, item in enumerate(obj)]

    def serialize_map(self, context: TypeEvaluationContext, tt: MapType, obj: Any) -> R:
        return {self._serialize_dispatch(context.append_path(f'[key {i}]'), tt.key_type, key):
                self._serialize_dispatch(context.append_path(f'[value {i}]'), tt.value_type, value)
                for i, (key, value) in enumerate(obj.items())}

    def serialize_record(self, context: TypeEvaluationContext, tt: RecordType, obj: Any) -> R:
        return {arg: self._serialize_dispatch(context.append_path(arg), arg_type, obj.get(arg))
                for arg, arg_type in tt.named_args}

    def serialize_variant(self, context: TypeEvaluationContext, tt: VariantType, obj: Any) -> R:
        obj_ctor, obj_value = decode_variant_dict(obj)
        value_ctor = tt.field_type(obj_ctor)

        new_value = self._serialize_dispatch(context.append_path(obj_ctor), value_ctor, obj_value)
        return {obj_ctor: new_value}

    def serialize_unsupported(
            self, context: TypeEvaluationContext, tt: UnsupportedType, obj: Any) -> R:
        return obj




# Copyright (c) 2019 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0


from asyncio import AbstractEventLoop, get_event_loop
from datetime import datetime, timedelta
from typing import Callable, Optional


class AioLoopPerfMonitor:

    def __init__(
            self,
            callback: 'Callable[[timedelta], None]',
            loop: 'Optional[AbstractEventLoop]' = None):
        self.callback = callback
        self.loop = loop
        self.started = False

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop()

    def start(self):
        loop = self.loop
        if loop is None:
            loop = get_event_loop()

        self.started = True
        loop.call_soon(self._tick, loop)

    def stop(self):
        self.started = False

    def _tick(self, loop: 'AbstractEventLoop') -> None:
        """
        Schedule a _tock.

        :param loop: The loop to use.
        """
        if self.started:
            loop.call_soon(self._tock, loop, datetime.utcnow())

    def _tock(self, loop: 'AbstractEventLoop', dt: 'datetime') -> None:
        """
        Measure the length of time from the last _tick and record that time; then schedule a _tock
        in one second.

        :param loop: The loop to use.
        :param dt: The time when the _tock was scheduled.
        """
        if self.started:
            now = datetime.utcnow()
            self.callback(now - dt)
            loop.call_later(1, self._tick, loop)



from typing import Collection

from aiohttp import web


def build_routes(prefix: str = '/metrics', registry=None) -> 'Collection[web.RouteDef]':
    routes = web.RouteTableDef()

    from prometheus_client import exposition, REGISTRY

    if registry is None:
        registry = REGISTRY

    @routes.get(prefix)
    async def get_metrics(request: 'web.Request') -> 'web.Response':
        encoder, content_type = exposition.choose_encoder(request.headers['Accept'])
        metrics_bytes: bytes = encoder(registry)
        return web.HTTPOk(body=metrics_bytes, content_type='text/plain; version=0.0.4')

    return routes



# Copyright (c) 2019 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
:mod:`dazl.damast` package
==========================
"""

from .evaluate import Evaluator
from .visitor import PackageVisitor, ModuleVisitor, ExprVisitor, TypeVisitor, IdentityTypeVisitor



# Copyright (c) 2019 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

import yaml
from dataclasses import dataclass
from pathlib import Path
from typing import Any, Dict, Mapping, Optional, Tuple

from semver import VersionInfo

from ..util.prim_types import to_boolean


@dataclass(frozen=True)
class ArtifactRepository:
    root_url: str
    artifacts: 'Dict[str, ArtifactLocation]'


@dataclass(frozen=True)
class ArtifactLocation:
    group_id: str
    artifact: str
    default_version: VersionInfo
    packaging: Optional[str] = None
    native: bool = False


def load_artifacts() -> 'ArtifactRepository':
    artifacts_yaml_file = Path(__file__).parent / 'artifacts.yaml'
    with artifacts_yaml_file.open() as f:
        d = yaml.load(f, yaml.SafeLoader)

    root_url = d.get('rootUrl')
    artifacts = d.get('artifacts')

    return ArtifactRepository(root_url, {key: _parse_artifact_dict(value)
                                         for key, value in artifacts.items()})


def _parse_artifact_dict(d: 'Mapping[str, Any]') -> 'ArtifactLocation':
    group_id = d.get('groupId')
    artifact = d.get('artifact')
    if group_id is None or artifact is None:
        raise ValueError('"groupId" and "artifact" must always be specified')

    native = d.get('native')
    if native is None:
        native = False
    else:
        try:
            native = to_boolean(native)
        except ValueError:
            raise ValueError(f'"native" must be a boolean (got {native!r} instead)')

    return ArtifactLocation(
        group_id, artifact, VersionInfo.parse(d.get('defaultVersion')),
        d.get('packaging'), native)


def parse_component_identifier(name: str, use_default_version: bool = False) \
        -> 'Tuple[str, Optional[VersionInfo]]':
    """
    Turn a string that looks like ``component`` or ``component:version`` into a tuple of component
    name and/or semver, optionally filling in the version value from the default taken from the
    manifest.

    :param name:
        The string that identifies the component, in the form ``component[:version]``.
    :param use_default_version:
        ``True`` to look in dazl's artifact manifest for a suggested version if none is supplied;
        otherwise, ``False`` to simply return ``None`` for version information.
    :return:
        A tuple of component name and optional version.
    :raise ValueError:
        ``name`` is not a valid component string

    >>> parse_component_identifier('sandbox')
    ('sandbox', None)
    >>> parse_component_identifier('sandbox:6.0.0')
    ('sandbox', VersionInfo(major=6, minor=0, patch=0, prerelease=None, build=None))
    """
    if name is None:
        raise ValueError('name is required')

    component, _, version = name.partition(':')
    if not component:
        raise ValueError('component name must be specified')

    parsed_version = VersionInfo.parse(version) if version else None
    if parsed_version is None:
        if use_default_version:
            parsed_version = load_artifacts().artifacts[component].default_version

    return component, parsed_version


def get_default_version(name: str) -> 'Optional[VersionInfo]':
    return load_artifacts().artifacts[name].default_version


def get_artifact_url(
        root_url: str,
        location: 'ArtifactLocation',
        version: 'Optional[VersionInfo]' = None,
        platform: 'Optional[str]' = None) -> str:
    requested_version = version or location.default_version
    if root_url.endswith('/'):
        root_url = root_url[:-1]

    group = location.group_id.replace('.', '/')
    ext = location.packaging or 'jar'
    if location.native:
        if platform is None:
            raise ValueError('platform is required for native binaries')
        file_name = f'{location.artifact}-{requested_version}-{platform}.{ext}'
    else:
        file_name = f'{location.artifact}-{requested_version}.{ext}'

    return f'{root_url}/{group}/{location.artifact}/{requested_version}/{file_name}'



# Copyright (c) 2019 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
This module contains the abstract base class that defines the protocol for interacting with a
process that implements the Ledger API.
"""
from asyncio import AbstractEventLoop, get_event_loop, Future, ensure_future
from concurrent.futures import Executor, ThreadPoolExecutor
from dataclasses import dataclass
from datetime import datetime, timedelta
from typing import Any, Optional, Sequence
import threading

from .. import LOG
from ..model.ledger import LedgerMetadata
from ..model.network import HTTPConnectionSettings
from ..model.reading import BaseEvent, TransactionFilter
from ..model.writing import CommandPayload
from ..util.typing import safe_optional_cast, safe_cast


@dataclass(frozen=True)
class LedgerConnectionOptions:
    connect_timeout: timedelta


class LedgerNetwork:
    """
    Abstract base class for creating connections to the ledger.
    """

    async def connect(
            self,
            party: str,
            settings: HTTPConnectionSettings,
            context_path: 'Optional[str]',
            admin_url: 'Optional[str]') -> 'LedgerClient':
        raise NotImplementedError('connect must be implemented')

    async def ledger(self) -> LedgerMetadata:
        """
        Return information about the entire ledger.
        """
        raise NotImplementedError('ledger must be implemented')

    async def set_time(self, new_datetime: datetime) -> None:
        """
        Advance the time on the Sandbox (static time model only).

        To access the current time, use the time_model returned from :meth:`ledger`.

        :param new_datetime: The new time to set.
        """
        raise NotImplementedError('set_time must be implemented')

    async def sync_time(self) -> None:
        """
        Ensure that the local notion of time is in sync with the remote server. This method is only
        relevant for static-time implementations on the REST-based API, as the gRPC API pushes time
        updates through the TimeService. In these contexts, this method is a no-op.
        """

    async def close(self):
        """
        Shut down all connections and transition the pool to the closed state.
        """

    @property
    def closed(self) -> bool:
        """
        Determine whether the underlying network connections to the LedgerNetwork are closed. This
        property should start as ``False`` when the pool is constructed, and ``True`` some time
        after ``close()`` is called.
        """
        raise NotImplementedError('closed must be implemented')


class LedgerClient:
    """
    Abstract base class that defines the required methods to define a protocol over the Ledger API.
    """

    async def commands(self, command_payload: CommandPayload) -> None:
        """
        Submit a command to the ledger.

        The coroutine returns when the implementation has accepted the command for processing, but
        not necessarily committed to the ledger.

        :param command_payload: Payload of data to submit asynchronously.
        """
        raise NotImplementedError('commands must be implemented')

    async def events(self, transaction_filter: TransactionFilter) -> Sequence[BaseEvent]:
        """
        Return events from a certain offset in the ledger. The number of blocks
        returned is implementation-defined.
        """
        raise NotImplementedError('events must be implemented')


class _LedgerConnectionContext:
    def __init__(self,
                 options: LedgerConnectionOptions,
                 loop: Optional[AbstractEventLoop] = None,
                 executor: Optional[Executor] = None):
        self.options = options
        self.loop = loop if loop is not None else get_event_loop()
        self.executor = executor if executor is not None else ThreadPoolExecutor(max_workers=25)

    def run_in_background(self, func, *args) -> Future:
        return ensure_future(self.loop.run_in_executor(self.executor, func, *args))

    def run_on_loop(self, func, *args) -> Any:
        """
        Called from a background thread to invoke a function on the event loop. This method blocks
        until the function has returned.
        """
        evt = threading.Event()
        result, ex = None, None

        def _invoke():
            nonlocal result
            try:
                result = func(*args)
            finally:
                evt.set()

        self.loop.call_soon_threadsafe(_invoke)
        evt.wait()
        return result


class _LedgerConnection:
    def __init__(self,
                 context: _LedgerConnectionContext,
                 settings: HTTPConnectionSettings,
                 context_path: Optional[str]):
        LOG.debug('Creating a gRPC channel for %s...', settings)
        import threading

        self.context = safe_cast(_LedgerConnectionContext, context)
        self.settings = safe_cast(HTTPConnectionSettings, settings)
        self.context_path = safe_optional_cast(str, context_path)
        self.close_evt = threading.Event()

    def close(self):
        self.close_evt.set()



# Copyright (c) 2019 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import ledger_configuration_service_pb2 as com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_ledger__configuration__service__pb2


class LedgerConfigurationServiceStub(object):
  """LedgerConfigurationService allows clients to subscribe to changes of the ledger configuration.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.GetLedgerConfiguration = channel.unary_stream(
        '/com.digitalasset.ledger.api.v1.LedgerConfigurationService/GetLedgerConfiguration',
        request_serializer=com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_ledger__configuration__service__pb2.GetLedgerConfigurationRequest.SerializeToString,
        response_deserializer=com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_ledger__configuration__service__pb2.GetLedgerConfigurationResponse.FromString,
        )


class LedgerConfigurationServiceServicer(object):
  """LedgerConfigurationService allows clients to subscribe to changes of the ledger configuration.
  """

  def GetLedgerConfiguration(self, request, context):
    """Returns the latest configuration as the first response, and publishes configuration updates in the same stream.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_LedgerConfigurationServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'GetLedgerConfiguration': grpc.unary_stream_rpc_method_handler(
          servicer.GetLedgerConfiguration,
          request_deserializer=com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_ledger__configuration__service__pb2.GetLedgerConfigurationRequest.FromString,
          response_serializer=com_dot_digitalasset_dot_ledger_dot_api_dot_v1_dot_ledger__configuration__service__pb2.GetLedgerConfigurationResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'com.digitalasset.ledger.api.v1.LedgerConfigurationService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))
